# Copyright 2022 Red Hat Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacServicesTests(rbac_base.ShareRbacBaseTests,
                             metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacServicesTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()

    def test_list_services(self):
        pass


class TestProjectAdminTests(ShareRbacServicesTests, base.BaseSharesTest):
    credentials = ['project_admin']

    @decorators.idempotent_id('08ec3a0b-6e4a-4cbf-bd15-3f48f8ddf71f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_services(self):
        self.do_request('list_services', expected_status=200)


class TestProjectMemberTests(ShareRbacServicesTests, base.BaseSharesTest):
    credentials = ['project_member']

    @decorators.idempotent_id('7431dca6-9b03-48d3-b97c-41f72f7ed0a3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_services(self):
        self.do_request('list_services', expected_status=lib_exc.Forbidden)


class TestProjectReaderTests(TestProjectMemberTests):
    credentials = ['project_reader']

    @decorators.idempotent_id('eca71619-d563-4d15-9e49-b661e6da46c0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_services(self):
        super(TestProjectReaderTests, self).test_list_services()



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import time
from urllib import parse as urlparse

from tempest import config
from tempest.lib.common import rest_client
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from manila_tempest_tests import share_exceptions


CONF = config.CONF


class SharesClient(rest_client.RestClient):
    """Tempest REST client for Manila.

    It handles shares and access to it in OpenStack.
    """

    def __init__(self, auth_provider, **kwargs):
        super(SharesClient, self).__init__(auth_provider, **kwargs)
        self.share_protocol = None
        if CONF.share.enable_protocols:
            self.share_protocol = CONF.share.enable_protocols[0]
        self.share_network_id = CONF.share.share_network_id
        self.share_size = CONF.share.share_size

    def _parse_resp(self, body, top_key_to_verify=None):
        return super(SharesClient, self)._parse_resp(
            body, top_key_to_verify=top_key_to_verify)

    def create_share(self, share_protocol=None, size=None,
                     name=None, snapshot_id=None, description=None,
                     metadata=None, share_network_id=None,
                     share_type_id=None, is_public=False):
        metadata = metadata or {}
        if name is None:
            name = data_utils.rand_name("tempest-created-share")
        if description is None:
            description = data_utils.rand_name("tempest-created-share-desc")
        if size is None:
            size = self.share_size
        if share_protocol is None:
            share_protocol = self.share_protocol
        if share_protocol is None:
            raise share_exceptions.ShareProtocolNotSpecified()
        post_body = {
            "share": {
                "share_proto": share_protocol,
                "description": description,
                "snapshot_id": snapshot_id,
                "name": name,
                "size": size,
                "metadata": metadata,
                "is_public": is_public,
            }
        }
        if share_network_id:
            post_body["share"]["share_network_id"] = share_network_id
        if share_type_id:
            post_body["share"]["share_type"] = share_type_id
        body = json.dumps(post_body)
        resp, body = self.post("shares", body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_share(self, share_id):
        resp, body = self.delete("shares/%s" % share_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def manage_share(self, service_host, protocol, export_path,
                     share_type_id, name=None, description=None):
        post_body = {
            "share": {
                "export_path": export_path,
                "service_host": service_host,
                "protocol": protocol,
                "share_type": share_type_id,
                "name": name,
                "description": description,
            }
        }
        body = json.dumps(post_body)
        resp, body = self.post("os-share-manage", body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def unmanage_share(self, share_id):
        resp, body = self.post(
            "os-share-unmanage/%s/unmanage" % share_id, None)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_shares(self, detailed=False, params=None):
        """Get list of shares w/o filters."""
        uri = 'shares/detail' if detailed else 'shares'
        uri += '?%s' % urlparse.urlencode(params) if params else ''
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_shares_with_detail(self, params=None):
        """Get detailed list of shares w/o filters."""
        return self.list_shares(detailed=True, params=params)

    def get_share(self, share_id):
        resp, body = self.get("shares/%s" % share_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_access_rule(self, share_id, access_type="ip",
                           access_to="0.0.0.0", access_level=None):
        post_body = {
            "os-allow_access": {
                "access_type": access_type,
                "access_to": access_to,
                "access_level": access_level,
            }
        }
        body = json.dumps(post_body)
        resp, body = self.post("shares/%s/action" % share_id, body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_access_rules(self, share_id):
        body = {"os-access_list": None}
        resp, body = self.post("shares/%s/action" % share_id, json.dumps(body))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_access_rule(self, share_id, rule_id):
        post_body = {
            "os-deny_access": {
                "access_id": rule_id,
            }
        }
        body = json.dumps(post_body)
        resp, body = self.post("shares/%s/action" % share_id, body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def extend_share(self, share_id, new_size):
        post_body = {
            "os-extend": {
                "new_size": new_size,
            }
        }
        body = json.dumps(post_body)
        resp, body = self.post("shares/%s/action" % share_id, body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def shrink_share(self, share_id, new_size):
        post_body = {
            "os-shrink": {
                "new_size": new_size,
            }
        }
        body = json.dumps(post_body)
        resp, body = self.post("shares/%s/action" % share_id, body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_snapshot(self, share_id, name=None, description=None,
                        force=False):
        if name is None:
            name = data_utils.rand_name("tempest-created-share-snap")
        if description is None:
            description = data_utils.rand_name(
                "tempest-created-share-snap-desc")
        post_body = {
            "snapshot": {
                "name": name,
                "force": force,
                "description": description,
                "share_id": share_id,
            }
        }
        body = json.dumps(post_body)
        resp, body = self.post("snapshots", body)
        self.expected_success(202, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_snapshot(self, snapshot_id):
        resp, body = self.get("snapshots/%s" % snapshot_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_snapshots(self, detailed=False, params=None):
        """Get list of share snapshots w/o filters."""
        uri = 'snapshots/detail' if detailed else 'snapshots'
        uri += '?%s' % urlparse.urlencode(params) if params else ''
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_snapshots_with_detail(self, params=None):
        """Get detailed list of share snapshots w/o filters."""
        return self.list_snapshots(detailed=True, params=params)

    def delete_snapshot(self, snap_id):
        resp, body = self.delete("snapshots/%s" % snap_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def default_quotas(self, tenant_id):
        resp, body = self.get("os-quota-sets/%s/defaults" % tenant_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_quotas(self, tenant_id, user_id=None):
        uri = "os-quota-sets/%s" % tenant_id
        if user_id is not None:
            uri += "?user_id=%s" % user_id
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def reset_quotas(self, tenant_id, user_id=None):
        uri = "os-quota-sets/%s" % tenant_id
        if user_id is not None:
            uri += "?user_id=%s" % user_id
        resp, body = self.delete(uri)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_quotas(self, tenant_id, user_id=None, shares=None,
                      snapshots=None, gigabytes=None, snapshot_gigabytes=None,
                      share_networks=None, force=True):
        uri = "os-quota-sets/%s" % tenant_id
        if user_id is not None:
            uri += "?user_id=%s" % user_id

        put_body = {"tenant_id": tenant_id}
        if force:
            put_body["force"] = "true"
        if shares is not None:
            put_body["shares"] = shares
        if snapshots is not None:
            put_body["snapshots"] = snapshots
        if gigabytes is not None:
            put_body["gigabytes"] = gigabytes
        if snapshot_gigabytes is not None:
            put_body["snapshot_gigabytes"] = snapshot_gigabytes
        if share_networks is not None:
            put_body["share_networks"] = share_networks
        put_body = json.dumps({"quota_set": put_body})

        resp, body = self.put(uri, put_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_limits(self):
        resp, body = self.get("limits")
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, *args, **kwargs):
        """Verifies whether provided resource deleted or not.

        :param kwargs: dict with expected keys 'share_id', 'snapshot_id',
        :param kwargs: 'sn_id', 'ss_id', 'vt_id' and 'server_id'
        :raises share_exceptions.InvalidResource
        """
        if "share_id" in kwargs:
            if "rule_id" in kwargs:
                rule_id = kwargs.get("rule_id")
                share_id = kwargs.get("share_id")
                rules = self.list_access_rules(share_id)['access_list']
                for rule in rules:
                    if rule["id"] == rule_id:
                        return False
                return True
            else:
                return self._is_resource_deleted(
                    self.get_share, kwargs.get("share_id"))
        elif "snapshot_id" in kwargs:
            return self._is_resource_deleted(
                self.get_snapshot, kwargs.get("snapshot_id"))
        elif "sn_id" in kwargs:
            return self._is_resource_deleted(
                self.get_share_network, kwargs.get("sn_id"))
        elif "ss_id" in kwargs:
            return self._is_resource_deleted(
                self.get_security_service, kwargs.get("ss_id"))
        elif "vt_id" in kwargs:
            return self._is_resource_deleted(
                self.get_volume_type, kwargs.get("vt_id"))
        elif "st_id" in kwargs:
            return self._is_resource_deleted(
                self.get_share_type, kwargs.get("st_id"))
        elif "server_id" in kwargs:
            return self._is_resource_deleted(
                self.show_share_server, kwargs.get("server_id"))
        elif "backup_id" in kwargs:
            return self._is_resource_deleted(
                self.get_share_backup, kwargs.get("backup_id"))
        else:
            raise share_exceptions.InvalidResource(
                message=str(kwargs))

    def _is_resource_deleted(self, func, res_id, **kwargs):
        try:
            res = func(res_id, **kwargs)
        except exceptions.NotFound:
            return True
        if res.get('status') in ['error_deleting', 'error']:
            # Resource has "error_deleting" status and can not be deleted.
            resource_type = func.__name__.split('_', 1)[-1]
            raise share_exceptions.ResourceReleaseFailed(
                res_type=resource_type, res_id=res_id)
        return False

    def wait_for_resource_deletion(self, *args, **kwargs):
        """Waits for a resource to be deleted."""
        start_time = int(time.time())
        while True:
            if self.is_resource_deleted(*args, **kwargs):
                return
            if int(time.time()) - start_time >= self.build_timeout:
                raise exceptions.TimeoutException
            time.sleep(self.build_interval)

    def list_extensions(self):
        resp, body = self.get("extensions")
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_share(self, share_id, name=None, desc=None, is_public=None):
        body = {"share": {}}
        if name is not None:
            body["share"].update({"display_name": name})
        if desc is not None:
            body["share"].update({"display_description": desc})
        if is_public is not None:
            body["share"].update({"is_public": is_public})
        body = json.dumps(body)
        resp, body = self.put("shares/%s" % share_id, body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def rename_snapshot(self, snapshot_id, name, desc=None):
        body = {"snapshot": {"display_name": name}}
        if desc is not None:
            body["snapshot"].update({"display_description": desc})
        body = json.dumps(body)
        resp, body = self.put("snapshots/%s" % snapshot_id, body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def reset_state(self, s_id, status="error", s_type="shares"):
        """Resets the state of a share or a snapshot.

        status: available, error, creating, deleting, error_deleting
        s_type: shares, snapshots
        """
        body = {"os-reset_status": {"status": status}}
        body = json.dumps(body)
        resp, body = self.post("%s/%s/action" % (s_type, s_id), body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def force_delete(self, s_id, s_type="shares"):
        """Force delete share or snapshot.

        s_type: shares, snapshots
        """
        body = {"os-force_delete": None}
        body = json.dumps(body)
        resp, body = self.post("%s/%s/action" % (s_type, s_id), body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

###############

    def list_services(self, params=None):
        """List services."""
        uri = 'os-services'
        if params:
            uri += '?%s' % urlparse.urlencode(params)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

###############

    def _update_metadata(self, share_id, metadata=None, method="post"):
        uri = "shares/%s/metadata" % share_id
        if metadata is None:
            metadata = {}
        post_body = {"metadata": metadata}
        body = json.dumps(post_body)
        if method == "post":
            resp, body = self.post(uri, body)
        if method == "put":
            resp, body = self.put(uri, body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def set_metadata(self, share_id, metadata=None):
        return self._update_metadata(share_id, metadata)

    def update_all_metadata(self, share_id, metadata=None):
        return self._update_metadata(share_id, metadata, method="put")

    def delete_metadata(self, share_id, key):
        resp, body = self.delete("shares/%s/metadata/%s" % (share_id, key))
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def get_metadata(self, share_id):
        resp, body = self.get("shares/%s/metadata" % share_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_metadata_item(self, share_id, key):
        resp, body = self.get("shares/%s/metadata/%s" % (share_id, key))
        self.expected_success(200, resp.status)
        return self._parse_resp(body, top_key_to_verify='meta')

###############

    def create_security_service(self, ss_type="ldap", **kwargs):
        # ss_type: ldap, kerberos, active_directory
        # kwargs: name, description, dns_ip, server, domain, user, password
        post_body = {"type": ss_type}
        post_body.update(kwargs)
        body = json.dumps({"security_service": post_body})
        resp, body = self.post("security-services", body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_security_service(self, ss_id, **kwargs):
        # ss_id - id of security-service entity
        # kwargs: dns_ip, server, domain, user, password, name, description
        # for 'active' status can be changed
        # only 'name' and 'description' fields
        body = json.dumps({"security_service": kwargs})
        resp, body = self.put("security-services/%s" % ss_id, body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_security_service(self, ss_id):
        resp, body = self.get("security-services/%s" % ss_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_security_services(self, detailed=False, params=None):
        uri = "security-services"
        if detailed:
            uri += '/detail'
        if params:
            uri += "?%s" % urlparse.urlencode(params)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_security_service(self, ss_id):
        resp, body = self.delete("security-services/%s" % ss_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

###############

    def create_share_network(self, **kwargs):
        # kwargs: name, description
        # + for neutron: neutron_net_id, neutron_subnet_id
        body = json.dumps({"share_network": kwargs})
        resp, body = self.post("share-networks", body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_share_network(self, sn_id, **kwargs):
        # kwargs: name, description
        # + for neutron: neutron_net_id, neutron_subnet_id
        body = json.dumps({"share_network": kwargs})
        resp, body = self.put("share-networks/%s" % sn_id, body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_share_network(self, sn_id):
        resp, body = self.get("share-networks/%s" % sn_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_share_networks(self):
        resp, body = self.get("share-networks")
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_share_networks_with_detail(self, params=None):
        """List the details of all shares."""
        uri = "share-networks/detail"
        if params:
            uri += "?%s" % urlparse.urlencode(params)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_share_network(self, sn_id):
        resp, body = self.delete("share-networks/%s" % sn_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

###############

    def _map_security_service_and_share_network(self, sn_id, ss_id,
                                                action="add"):
        # sn_id: id of share_network_entity
        # ss_id: id of security service entity
        # action: add, remove
        data = {
            "%s_security_service" % action: {
                "security_service_id": ss_id,
            }
        }
        body = json.dumps(data)
        resp, body = self.post("share-networks/%s/action" % sn_id, body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def add_sec_service_to_share_network(self, sn_id, ss_id):
        return self._map_security_service_and_share_network(sn_id, ss_id)

    def remove_sec_service_from_share_network(self, sn_id, ss_id):
        return self._map_security_service_and_share_network(
            sn_id, ss_id, "remove")

    def list_sec_services_for_share_network(self, sn_id):
        resp, body = self.get("security-services?share_network_id=%s" % sn_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

###############

    def list_share_types(self, params=None):
        uri = 'types'
        if params is not None:
            uri += '?%s' % urlparse.urlencode(params)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_share_type(self, name, is_public=True, **kwargs):
        post_body = {
            'name': name,
            'extra_specs': kwargs.get('extra_specs'),
            'os-share-type-access:is_public': is_public,
        }
        post_body = json.dumps({'share_type': post_body})
        resp, body = self.post('types', post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_share_type(self, share_type_id):
        resp, body = self.delete("types/%s" % share_type_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def get_share_type(self, share_type_id):
        resp, body = self.get("types/%s" % share_type_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_default_share_type(self):
        resp, body = self.get("types/default")
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def add_access_to_share_type(self, share_type_id, project_id):
        uri = 'types/%s/action' % share_type_id
        post_body = {'project': project_id}
        post_body = json.dumps({'addProjectAccess': post_body})
        resp, body = self.post(uri, post_body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def remove_access_from_share_type(self, share_type_id, project_id):
        uri = 'types/%s/action' % share_type_id
        post_body = {'project': project_id}
        post_body = json.dumps({'removeProjectAccess': post_body})
        resp, body = self.post(uri, post_body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_access_to_share_type(self, share_type_id):
        uri = 'types/%s/os-share-type-access' % share_type_id
        resp, body = self.get(uri)
        # [{"share_type_id": "%st_id%", "project_id": "%project_id%"}, ]
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

###############

    def create_share_type_extra_specs(self, share_type_id, extra_specs):
        url = "types/%s/extra_specs" % share_type_id
        post_body = json.dumps({'extra_specs': extra_specs})
        resp, body = self.post(url, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_share_type_extra_spec(self, share_type_id, extra_spec_name):
        uri = "types/%s/extra_specs/%s" % (share_type_id, extra_spec_name)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_share_type_extra_specs(self, share_type_id, params=None):
        uri = "types/%s/extra_specs" % share_type_id
        if params is not None:
            uri += '?%s' % urlparse.urlencode(params)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_share_type_extra_spec(self, share_type_id, spec_name,
                                     spec_value):
        uri = "types/%s/extra_specs/%s" % (share_type_id, spec_name)
        extra_spec = {spec_name: spec_value}
        post_body = json.dumps(extra_spec)
        resp, body = self.put(uri, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_share_type_extra_specs(self, share_type_id, extra_specs):
        uri = "types/%s/extra_specs" % share_type_id
        extra_specs = {"extra_specs": extra_specs}
        post_body = json.dumps(extra_specs)
        resp, body = self.post(uri, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_share_type_extra_spec(self, share_type_id, extra_spec_name):
        uri = "types/%s/extra_specs/%s" % (share_type_id, extra_spec_name)
        resp, body = self.delete(uri)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

###############

    def list_share_servers(self, search_opts=None):
        """Get list of share servers."""
        uri = "share-servers"
        if search_opts:
            uri += "?%s" % urlparse.urlencode(search_opts)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_share_server(self, share_server_id):
        """Delete share server by its ID."""
        uri = "share-servers/%s" % share_server_id
        resp, body = self.delete(uri)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_share_server(self, share_server_id):
        """Get share server info."""
        uri = "share-servers/%s" % share_server_id
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_share_server_details(self, share_server_id):
        """Get share server details only."""
        uri = "share-servers/%s/details" % share_server_id
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

###############

    def list_pools(self, detail=False, search_opts=None):
        """Get list of scheduler pools."""
        uri = 'scheduler-stats/pools'
        if detail:
            uri += '/detail'
        if search_opts:
            uri += "?%s" % urlparse.urlencode(search_opts)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

###############

    def list_availability_zones(self):
        """Get list of availability zones."""
        uri = 'os-availability-zone'
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF


@ddt.ddt
class RevertToSnapshotTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(RevertToSnapshotTest, cls).skip_checks()
        if not CONF.share.run_revert_to_snapshot_tests:
            msg = "Revert to snapshot tests are disabled."
            raise cls.skipException(msg)
        if not CONF.share.capability_snapshot_support:
            msg = "Snapshot support is disabled."
            raise cls.skipException(msg)
        if not CONF.share.run_snapshot_tests:
            msg = "Snapshot tests are disabled."
            raise cls.skipException(msg)

        utils.check_skip_if_microversion_not_supported(
            constants.REVERT_TO_SNAPSHOT_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(RevertToSnapshotTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.replication_multitenancy = (
            utils.replication_with_multitenancy_support())
        pools = cls.admin_client.list_pools(detail=True)['pools']
        revert_support = [
            pool['capabilities'][constants.REVERT_TO_SNAPSHOT_SUPPORT]
            for pool in pools]
        if not any(revert_support):
            msg = "Revert to snapshot not supported."
            raise cls.skipException(msg)

        cls.share_type_name = data_utils.rand_name("share-type")
        extra_specs = {
            "snapshot_support": True,
            constants.REVERT_TO_SNAPSHOT_SUPPORT: True,
        }
        cls.revert_enabled_extra_specs = cls.add_extra_specs_to_dict(
            extra_specs=extra_specs)

        cls.share_type = cls.create_share_type(
            cls.share_type_name,
            extra_specs=cls.revert_enabled_extra_specs,
            client=cls.admin_client)

        cls.st_id = cls.share_type['id']

        cls.share = cls.create_share(share_type_id=cls.st_id)

        if CONF.share.run_replication_tests:
            # Create replicated share type
            cls.replicated_share_type_name = data_utils.rand_name("share-type")
            cls.replication_type = CONF.share.backend_replication_type
            if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
                raise share_exceptions.ShareReplicationTypeException(
                    replication_type=cls.replication_type
                )
            extra_specs = cls.add_extra_specs_to_dict({
                "replication_type": cls.replication_type,
                "snapshot_support": True,
                constants.REVERT_TO_SNAPSHOT_SUPPORT: True,
            })
            cls.replicated_share_type = cls.create_share_type(
                cls.replicated_share_type_name,
                extra_specs=extra_specs,
                client=cls.admin_client)
            cls.zones = cls.get_availability_zones_matching_share_type(
                cls.replicated_share_type, client=cls.admin_client)
            cls.share_zone = cls.zones[0]
            cls.replica_zone = cls.zones[-1]
            cls.sn_id = None
            if cls.replication_multitenancy:
                cls.share_network = cls.shares_v2_client.get_share_network(
                    cls.shares_v2_client.share_network_id)['share_network']
                cls.sn_id = cls.share_network['id']

    @decorators.idempotent_id('196f2bc5-e13a-4730-ac51-61e339068a06')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_to_latest_snapshot(self, version):
        snapshot = self.create_snapshot_wait_for_active(self.share['id'],
                                                        cleanup_in_class=False)
        self.shares_v2_client.revert_to_snapshot(
            self.share['id'],
            snapshot['id'],
            version=version)
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.share['id'],
            constants.STATUS_AVAILABLE)
        waiters.wait_for_resource_status(
            self.shares_v2_client, snapshot['id'], constants.STATUS_AVAILABLE,
            resource_name='snapshot')

    @decorators.idempotent_id('09bd9942-7ef9-4d24-b2dd-f83bdda27b50')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_to_previous_snapshot(self, version):
        snapshot1 = self.create_snapshot_wait_for_active(
            self.share['id'], cleanup_in_class=False)
        snapshot2 = self.create_snapshot_wait_for_active(
            self.share['id'], cleanup_in_class=False)

        self.shares_v2_client.delete_snapshot(snapshot2['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            snapshot_id=snapshot2['id'])

        self.shares_v2_client.revert_to_snapshot(self.share['id'],
                                                 snapshot1['id'],
                                                 version=version)
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.share['id'],
            constants.STATUS_AVAILABLE)
        waiters.wait_for_resource_status(
            self.shares_v2_client, snapshot1['id'], constants.STATUS_AVAILABLE,
            resource_name='snapshot')

    @decorators.idempotent_id('146de138-d351-49dc-a13a-5cdbed40b9ac')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @tc.skipUnless(CONF.share.run_replication_tests,
                   'Replication tests are disabled.')
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_to_replicated_snapshot(self, version):
        """Test reverting to a replicated snapshot."""
        share = self.create_share(
            share_type_id=self.replicated_share_type['id'],
            availability_zone=self.share_zone,
            share_network_id=self.sn_id
        )

        share_replica = self.create_share_replica(share["id"],
                                                  self.replica_zone)
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

        snapshot = self.create_snapshot_wait_for_active(share["id"])

        self.shares_v2_client.revert_to_snapshot(
            share['id'],
            snapshot['id'],
            version=version)
        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'], constants.STATUS_AVAILABLE)
        waiters.wait_for_resource_status(
            self.shares_v2_client, share_replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')
        waiters.wait_for_resource_status(
            self.shares_v2_client, snapshot['id'], constants.STATUS_AVAILABLE,
            resource_name='snapshot')



# Copyright 2019 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ManageNFSShareNegativeTest(base.BaseSharesAdminTest):
    protocol = 'nfs'

    # NOTE(lseki): be careful running these tests using generic driver
    # because cinder volumes will stay attached to service Nova VM and
    # won't be deleted.

    @classmethod
    def skip_checks(cls):
        super(ManageNFSShareNegativeTest, cls).skip_checks()
        if not CONF.share.run_manage_unmanage_tests:
            raise cls.skipException('Manage/unmanage tests are disabled.')
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

        utils.skip_if_manage_not_supported_for_version()

    @classmethod
    def resource_setup(cls):
        super(ManageNFSShareNegativeTest, cls).resource_setup()
        # Create share type
        cls.st_name = data_utils.rand_name(name="manage-st-name")
        cls.extra_specs = {
            'storage_protocol': CONF.share.capability_storage_protocol,
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
            'snapshot_support': CONF.share.capability_snapshot_support,
        }

        cls.st = cls.create_share_type(
            name=cls.st_name,
            cleanup_in_class=True,
            extra_specs=cls.extra_specs)

    def _manage_share_and_wait(self, params,
                               state=constants.STATUS_AVAILABLE):
        # Manage the share and wait for the expected state.
        # Return the managed share object.
        managed_share = self.shares_v2_client.manage_share(**params)['share']
        waiters.wait_for_resource_status(
            self.shares_v2_client, managed_share['id'], state)

        return managed_share

    def _get_manage_params_from_share(self, share, invalid_params=None):
        valid_params = {
            'service_host': share['host'],
            'protocol': share['share_proto'],
            'share_type_id': share['share_type'],
        }
        if CONF.share.multitenancy_enabled:
            valid_params['share_server_id'] = share['share_server_id']

        if utils.is_microversion_ge(CONF.share.max_api_microversion, "2.9"):
            el = self.shares_v2_client.list_share_export_locations(
                share["id"])['export_locations']
            valid_params['export_path'] = el[0]['path']

        if invalid_params:
            valid_params.update(invalid_params)

        return valid_params

    @decorators.idempotent_id('8267161e-4f55-44eb-9af5-30d1a3fb2606')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_invalid_param_raises_exception(self):
        # Try to manage share with invalid parameters, it should not succeed
        # because the api will reject it. If it succeeds, then this test case
        # failed. Then, in order to remove the resource from backend, we need
        # to manage it again, properly, so we can delete it. Consequently the
        # second part of this test also tests that manage operation with a
        # proper share type that works.

        share = self._create_share_for_manage()

        valid_params = self._get_manage_params_from_share(share)
        self._unmanage_share_and_wait(share)

        test_set = [
            ('service_host', 'invalid_host#invalid_pool', lib_exc.NotFound),
            ('share_type_id', 'invalid_share_type_id', lib_exc.NotFound),
        ]
        if CONF.share.multitenancy_enabled:
            test_set.append(
                ('share_server_id', 'invalid_server_id', lib_exc.BadRequest)
            )

        for invalid_key, invalid_value, expected_exception in test_set:
            # forge a bad param
            invalid_params = valid_params.copy()
            invalid_params.update({
                invalid_key: invalid_value
            })

            # the attempt to manage with bad param should fail and raise an
            # exception
            self.assertRaises(
                expected_exception,
                self.shares_v2_client.manage_share,
                **invalid_params
            )

        # manage it properly and cleanup
        managed_share = self._manage_share_and_wait(valid_params)
        self._delete_share_and_wait(managed_share)

        # Delete share server, since it can't be "auto-deleted"
        if (CONF.share.multitenancy_enabled and
                not CONF.share.share_network_id):
            # For a pre-configured share_network_id, we don't
            # delete the share server.
            self._delete_share_server_and_wait(
                managed_share['share_server_id'])

    @decorators.idempotent_id('01b36343-c503-4c14-bb74-71f131ae96b6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_invalid_param_manage_error(self):
        # Try to manage share with invalid parameters, it should not succeed.
        # If it succeeds, then this test case failed. Then, in order to remove
        # the resource from backend, we need to manage it again, properly, so
        # we can delete it. Consequently the second part of this test also
        # tests that manage operation with a proper share type works.
        share = self._create_share_for_manage()

        valid_params = self._get_manage_params_from_share(share)
        self._unmanage_share_and_wait(share)

        for invalid_key, invalid_value in (
            ('export_path',
             data_utils.rand_name(name='invalid-share-export')),
            ('protocol', 'invalid_protocol'),
        ):

            # forge a bad param
            invalid_params = valid_params.copy()
            invalid_params.update({invalid_key: invalid_value})

            # the attempt to manage the share with invalid params should fail
            # and leave it in manage_error state
            invalid_share = self.shares_v2_client.manage_share(
                **invalid_params
            )['share']
            waiters.wait_for_resource_status(
                self.shares_v2_client, invalid_share['id'],
                constants.STATUS_MANAGE_ERROR)

            # cleanup
            self._unmanage_share_and_wait(invalid_share)

        # manage it properly and cleanup
        managed_share = self._manage_share_and_wait(valid_params)
        self._delete_share_and_wait(managed_share)

        # Delete share server, since it can't be "auto-deleted"
        if (CONF.share.multitenancy_enabled and
                not CONF.share.share_network_id):
            # For a pre-configured share_network_id, we don't
            # delete the share server.
            self._delete_share_server_and_wait(
                managed_share['share_server_id'])

    @decorators.idempotent_id('832b1c4d-e6f2-4a75-9ea5-d99f780db743')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share_duplicate(self):
        share = self._create_share_for_manage()

        manage_params = self._get_manage_params_from_share(share)
        self._unmanage_share_and_wait(share)

        # manage the share for the first time
        managed_share = self._manage_share_and_wait(manage_params)

        # update managed share's reference
        managed_share = self.shares_v2_client.get_share(
            managed_share['id'])['share']
        manage_params = self._get_manage_params_from_share(managed_share)

        # the second attempt to manage the same share should fail
        self.assertRaises(
            lib_exc.Conflict,
            self.shares_v2_client.manage_share,
            **manage_params
        )

        # cleanup
        self._delete_share_and_wait(managed_share)

        # Delete share server, since it can't be "auto-deleted"
        if (CONF.share.multitenancy_enabled and
                not CONF.share.share_network_id):
            # For a pre-configured share_network_id, we don't
            # delete the share server.
            self._delete_share_server_and_wait(
                managed_share['share_server_id'])

    @decorators.idempotent_id('c2f28981-5758-46e8-9d90-5c3269461d94')
    @testtools.skipUnless(CONF.share.multitenancy_enabled,
                          'Multitenancy tests are disabled.')
    @utils.skip_if_microversion_not_supported("2.49")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share_without_share_server_id(self):
        share = self._create_share_for_manage()
        manage_params = self._get_manage_params_from_share(share)
        share_server_id = manage_params.pop('share_server_id')
        self._unmanage_share_and_wait(share)

        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.manage_share,
            **manage_params)

        manage_params['share_server_id'] = share_server_id

        managed_share = self._manage_share_and_wait(manage_params)
        self._delete_share_and_wait(managed_share)

        # Delete share server, since it can't be "auto-deleted"
        if (CONF.share.multitenancy_enabled and
                not CONF.share.share_network_id):
            # For a pre-configured share_network_id, we don't
            # delete the share server.
            self._delete_share_server_and_wait(
                managed_share['share_server_id'])

    @decorators.idempotent_id('162b6b96-e801-40f4-b111-7e533cf8b943')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share_in_manage_error(self):
        share = self._create_share_for_manage()

        valid_params = self._get_manage_params_from_share(share)

        # forge bad param to have a share in manage_error state
        invalid_params = valid_params.copy()
        invalid_params.update(
            {'export_path': data_utils.rand_name(name='invalid-share-export')}
        )
        invalid_share = self.shares_v2_client.manage_share(
            **invalid_params)['share']

        waiters.wait_for_resource_status(
            self.shares_v2_client, invalid_share['id'],
            constants.STATUS_MANAGE_ERROR)
        self._unmanage_share_and_wait(share)

        # the attempt to delete a share in manage_error should raise an
        # exception
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.delete_share,
            invalid_share['id']
        )

        # cleanup
        self.shares_v2_client.unmanage_share(invalid_share['id'])
        managed_share = self._manage_share_and_wait(valid_params)
        self._delete_share_and_wait(managed_share)

        # Delete share server, since it can't be "auto-deleted"
        if (CONF.share.multitenancy_enabled and
                not CONF.share.share_network_id):
            # For a pre-configured share_network_id, we don't
            # delete the share server.
            self._delete_share_server_and_wait(
                managed_share['share_server_id'])

    @decorators.idempotent_id('7cad4763-8f78-46a5-aeeb-6ca174284d3d')
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          'Snapshot tests are disabled.')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share_with_snapshot(self):
        # A share with snapshot cannot be unmanaged
        share = self._create_share_for_manage()

        snap = self.create_snapshot_wait_for_active(share["id"])
        snap = self.shares_v2_client.get_snapshot(snap['id'])['snapshot']

        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.unmanage_share,
            share['id']
        )

        # cleanup
        self._delete_snapshot_and_wait(snap)
        self._delete_share_and_wait(share)

    @decorators.idempotent_id('f71bcb3c-836e-4e9e-972a-50debeb8c285')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share_transitional_state(self):
        # A share in transitional state cannot be unmanaged
        share = self._create_share_for_manage()
        for state in (constants.STATUS_CREATING,
                      constants.STATUS_DELETING,
                      constants.STATUS_MIGRATING,
                      constants.STATUS_MIGRATING_TO):
            self.shares_v2_client.reset_state(share['id'], state)

            self.assertRaises(
                lib_exc.Forbidden,
                self.shares_v2_client.unmanage_share,
                share['id']
            )

        # cleanup
        self._reset_state_and_delete_share(share)

    @decorators.idempotent_id('916ca8e2-5635-4cc9-8ab6-36b7b53ecd0a')
    @testtools.skipUnless(CONF.share.multitenancy_enabled,
                          'Multitenancy tests are disabled.')
    @utils.skip_if_microversion_not_supported("2.48")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share_with_server_unsupported(self):
        share = self._create_share_for_manage()

        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.unmanage_share,
            share['id'], version="2.48")

        self._delete_share_and_wait(share)


class ManageCIFSShareNegativeTest(ManageNFSShareNegativeTest):
    protocol = 'cifs'


class ManageGLUSTERFSShareNegativeTest(ManageNFSShareNegativeTest):
    protocol = 'glusterfs'


class ManageHDFSShareNegativeTest(ManageNFSShareNegativeTest):
    protocol = 'hdfs'


class ManageCephFSShareNegativeTest(ManageNFSShareNegativeTest):
    protocol = 'cephfs'


class ManageMapRFSShareNegativeTest(ManageNFSShareNegativeTest):
    protocol = 'maprfs'



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


class ShareLimitsTest(base.BaseSharesTest):

    @decorators.idempotent_id('239903d2-f1cb-4bec-b456-d57456308244')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_limits_keys(self):

        # list limits
        limits = self.shares_client.get_limits()['limits']

        # verify response
        keys = ["rate", "absolute"]
        [self.assertIn(key, limits.keys()) for key in keys]

        abs_keys = [
            "maxTotalShareGigabytes",
            "maxTotalShares",
            "maxTotalShareSnapshots",
            "maxTotalShareNetworks",
            "maxTotalSnapshotGigabytes",
            "totalSharesUsed",
            "totalShareSnapshotsUsed",
            "totalShareNetworksUsed",
            "totalShareGigabytesUsed",
            "totalSnapshotGigabytesUsed",
        ]
        [self.assertIn(key, limits["absolute"].keys()) for key in abs_keys]

    @decorators.idempotent_id('fea1a10e-6bcb-46eb-bbba-f6f0a8d4f377')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_limits_values(self):

        # list limits
        limits = self.shares_client.get_limits()['limits']

        # verify integer values for absolute limits
        abs_l = limits["absolute"]
        self.assertGreater(int(abs_l["maxTotalShareGigabytes"]), -2)
        self.assertGreater(int(abs_l["maxTotalShares"]), -2)
        self.assertGreater(int(abs_l["maxTotalShareSnapshots"]), -2)
        self.assertGreater(int(abs_l["maxTotalShareNetworks"]), -2)
        self.assertGreater(int(abs_l["maxTotalSnapshotGigabytes"]), -2)
        self.assertGreater(int(abs_l["totalSharesUsed"]), -2)
        self.assertGreater(int(abs_l["totalShareSnapshotsUsed"]), -2)
        self.assertGreater(int(abs_l["totalShareNetworksUsed"]), -2)
        self.assertGreater(int(abs_l["totalShareGigabytesUsed"]), -2)
        self.assertGreater(int(abs_l["totalSnapshotGigabytesUsed"]), -2)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.scenario import manager_share as manager
from manila_tempest_tests import utils


CONF = config.CONF
LOG = logging.getLogger(__name__)


@ddt.ddt
class ShareManageUnmanageBase(manager.ShareScenarioTest):

    """This test case uses the following flow:

     * Launch an instance
     * Create share (1GB)
     * Configure RW access to the share
     * Perform ssh to instance
     * Mount share
     * Write data in share
     * Unmount share
     * Unmanage share
     * Attempt to access share (fail expected)
     * Manage share
     * Configure RW access to the share
     * Mount share
     * Read data from share
     * Unmount share
     * Delete share
     * Attempt to manage share (fail expected)
     * Delete failed managed share
     * Terminate the instance
    """

    @classmethod
    def skip_checks(cls):
        super(ShareManageUnmanageBase, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_ip_rules_for_protocols:
            message = ("%s tests for access rules other than IP are disabled" %
                       cls.protocol)
            raise cls.skipException(message)

    @decorators.idempotent_id('165f22b4-9417-45ee-8e1d-2b8669ac2f00')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_manage_unmanage_tests,
        "Manage/unmanage tests are disabled.")
    @testtools.skipIf(
        CONF.share.multitenancy_enabled,
        "Manage/unmanage tests are skipped when DHSS is enabled")
    def test_create_manage_and_write(self):
        share_size = CONF.share.share_size

        LOG.debug('Step 1 - create instance')
        instance = self.boot_instance(wait_until="BUILD")

        LOG.debug('Step 2 - create share of size {} Gb'.format(share_size))
        share = self.create_share(size=share_size, cleanup=False)
        instance = self.wait_for_active_instance(instance["id"])

        LOG.debug('Step 3 - SSH to UVM')
        remote_client = self.init_remote_client(instance)

        LOG.debug('Step 4 - provide access to instance')
        self._provide_access_to_client_identified_by_ip(instance, share=share)

        if utils.is_microversion_lt(CONF.share.max_api_microversion, "2.9"):
            locations = share['export_locations']
        else:
            exports = self.shares_v2_client.list_share_export_locations(
                share['id'])['export_locations']
            locations = [x['path'] for x in exports]

        LOG.debug('Step 5 - mount')
        self.mount_share(locations[0], remote_client)

        # Update share info, needed later
        share = self.shares_admin_v2_client.get_share(share['id'])['share']

        LOG.debug('Step 6a - create file')
        remote_client.exec_command("sudo touch /mnt/t1")

        LOG.debug('Step 6b - write data')
        LOG.debug('Step 6b - writing 640mb')
        self.write_data_to_mounted_share_using_dd(remote_client,
                                                  '/mnt/t1', 1024,
                                                  2048)
        ls_result = remote_client.exec_command("sudo ls -lA /mnt/")
        LOG.debug(ls_result)

        LOG.debug('Step 7 - unmount share')
        self.unmount_share(remote_client)

        LOG.debug('Step 8a - unmanage share')
        self.shares_admin_v2_client.unmanage_share(share['id'])

        LOG.debug('Step 8b - wait for status change')
        self.shares_admin_v2_client.wait_for_resource_deletion(
            share_id=share['id'])

        LOG.debug('Step 9 - get share, should fail')
        self.assertRaises(
            exceptions.NotFound,
            self.shares_admin_v2_client.get_share,
            share['id'])

        LOG.debug('Step 10 - manage share')
        share_type = self.get_share_type()
        managed_share = self.shares_admin_v2_client.manage_share(
            share['host'],
            share['share_proto'],
            locations[0],
            share_type['id'])['share']
        waiters.wait_for_resource_status(
            self.shares_admin_v2_client, managed_share['id'], 'available')

        LOG.debug('Step 11 - grant access again')
        self._provide_access_to_client_identified_by_ip(
            instance,
            share=managed_share,
            client=self.shares_admin_v2_client)

        exports = self.shares_admin_v2_client.list_share_export_locations(
            managed_share['id'])['export_locations']
        locations = [x['path'] for x in exports]

        LOG.debug('Step 12 - mount')
        self.mount_share(locations[0], remote_client)

        LOG.debug('Step 12 - verify data')
        ls_result = remote_client.exec_command("sudo ls -lA /mnt/")
        LOG.debug(ls_result)

        LOG.debug('Step 13 - unmount share')
        self.unmount_share(remote_client)

        LOG.debug('Step 14 - delete share')
        self.shares_admin_v2_client.delete_share(managed_share['id'])
        self.shares_admin_v2_client.wait_for_resource_deletion(
            share_id=managed_share['id'])

        LOG.debug('Step 15 - manage share, should fail')
        remanaged_share = self.shares_admin_v2_client.manage_share(
            share['host'],
            share['share_proto'],
            locations[0],
            share_type['id'])['share']
        waiters.wait_for_resource_status(
            self.shares_admin_v2_client, remanaged_share['id'], 'manage_error')

        self.shares_admin_v2_client.reset_state(remanaged_share['id'])

        LOG.debug('Step 16 - delete failed managed share')
        self.shares_admin_v2_client.delete_share(remanaged_share['id'])
        self.shares_admin_v2_client.wait_for_resource_deletion(
            share_id=remanaged_share['id'])


class ShareManageUnmanageNFS(manager.BaseShareScenarioNFSTest,
                             ShareManageUnmanageBase):
    pass


class ShareManageUnmanageCIFS(manager.BaseShareScenarioCIFSTest,
                              ShareManageUnmanageBase):
    pass


class ShareManageUnmanageNFSIPv6(ShareManageUnmanageNFS):
    ip_version = 6


# NOTE(u_glide): this function is required to exclude ShareManageUnmanageBase
# from executed test cases.
# See: https://docs.python.org/3/library/unittest.html#load-tests-protocol
# for details.
def load_tests(loader, tests, _):
    result = []
    for test_case in tests:
        if type(test_case._tests[0]) is ShareManageUnmanageBase:
            continue
        result.append(test_case)
    return loader.suiteClass(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ShareSnapshotMetadataNegativeTest(base.BaseSharesMixedTest):
    @classmethod
    def skip_checks(cls):
        super(ShareSnapshotMetadataNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.73")
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')

    @classmethod
    def resource_setup(cls):
        super(ShareSnapshotMetadataNegativeTest, cls).resource_setup()
        # create share_type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        cls.metadata = {
            'foo_key_share_1': 'foo_value_share_1',
            'bar_key_share_1': 'foo_value_share_1',
        }
        cls.share = cls.create_share(
            name=cls.share_name,
            description=cls.share_desc,
            metadata=cls.metadata,
            share_type_id=cls.share_type_id,
        )
        cls.share_id = cls.share["id"]

        # create snapshot
        cls.snap_name = data_utils.rand_name("tempest-snapshot-name")
        cls.snap_desc = data_utils.rand_name(
            "tempest-snapshot-description")
        cls.snap = cls.create_snapshot_wait_for_active(
            cls.share_id, cls.snap_name, cls.snap_desc)
        cls.snap_id = cls.snap['id']

    @decorators.idempotent_id('8be4773b-6af9-413f-97e2-8acdb6149e7a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_metadata_to_unexisting_snapshot(self):
        md = {u"key1": u"value1", u"key2": u"value2", }
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.set_metadata,
                          "wrong_snapshot_id", md, resource="snapshot")

    @decorators.idempotent_id('03a7f6e9-de8b-4669-87e1-b179308b477d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_all_metadata_to_unexisting_snapshot(self):
        md = {u"key1": u"value1", u"key2": u"value2", }
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.update_all_metadata,
                          "wrong_snapshot_id", md, resource="snapshot")

    @decorators.idempotent_id('ef0afcc8-7b12-41bc-8aa1-4916ad4b4560')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_set_metadata_with_empty_key(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.snap_id, {"": "value"}, resource="snapshot")

    @decorators.idempotent_id('9f2aee7c-ebd6-4516-87f7-bd85453d74c9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_upd_metadata_with_empty_key(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.snap_id, {"": "value"}, resource="snapshot")

    @decorators.idempotent_id('ef61255e-462c-49fe-8e94-ff3afafcccb3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_set_metadata_with_too_big_key(self):
        too_big_key = "x" * 256
        md = {too_big_key: "value"}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.snap_id, md, resource="snapshot")

    @decorators.idempotent_id('f896f354-6179-4abb-b0c5-7b7dc96f0870')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_upd_metadata_with_too_big_key(self):
        too_big_key = "x" * 256
        md = {too_big_key: "value"}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.snap_id, md, resource="snapshot")

    @decorators.idempotent_id('1bf97c18-27df-4618-94f4-224d1a98bc0c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_set_metadata_with_too_big_value(self):
        too_big_value = "x" * 1024
        md = {"key": too_big_value}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.set_metadata,
                          self.snap_id, md, resource="snapshot")

    @decorators.idempotent_id('2b9e08fa-b35d-4bfe-9137-e59ab50bd9ef')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_upd_metadata_with_too_big_value(self):
        too_big_value = "x" * 1024
        md = {"key": too_big_value}
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.update_all_metadata,
                          self.snap_id, md, resource="snapshot")

    @decorators.idempotent_id('9afb381d-c48c-4c2c-a5b5-42463daef5a2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_try_delete_unexisting_metadata(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_metadata,
                          self.snap_id, "wrong_key", resource="snapshot")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ReplicationExportLocationsTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ReplicationExportLocationsTest, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')

    @classmethod
    def resource_setup(cls):
        super(ReplicationExportLocationsTest, cls).resource_setup()
        # Create share_type
        name = data_utils.rand_name(constants.TEMPEST_MANILA_PREFIX)
        cls.admin_client = cls.admin_shares_v2_client
        cls.replication_type = CONF.share.backend_replication_type
        cls.multitenancy_enabled = (
            utils.replication_with_multitenancy_support())

        if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=cls.replication_type
            )
        cls.extra_specs = cls.add_extra_specs_to_dict(
            {"replication_type": cls.replication_type})
        cls.share_type = cls.create_share_type(
            name,
            extra_specs=cls.extra_specs,
            client=cls.admin_client)
        cls.sn_id = None
        if cls.multitenancy_enabled:
            cls.share_network = cls.shares_v2_client.get_share_network(
                cls.shares_v2_client.share_network_id)['share_network']
            cls.sn_id = cls.share_network['id']
        cls.zones = cls.get_availability_zones_matching_share_type(
            cls.share_type)
        cls.share_zone = cls.zones[0]
        cls.replica_zone = cls.zones[-1]

    @staticmethod
    def _remove_admin_only_exports(all_exports):
        return [e for e in all_exports if not e['is_admin_only']]

    def _create_share_and_replica_get_exports(self, cleanup_replica=True):
        share = self.create_share(share_type_id=self.share_type['id'],
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id)
        replica = self.create_share_replica(share['id'], self.replica_zone,
                                            cleanup=cleanup_replica)
        replicas = self.shares_v2_client.list_share_replicas(
            share_id=share['id'])['share_replicas']
        primary_replica = [r for r in replicas if r['id'] != replica['id']][0]

        # Refresh share and replica
        share = self.shares_v2_client.get_share(share['id'])['share']
        replica = self.shares_v2_client.get_share_replica(
            replica['id'])['share_replica']

        # Grab export locations of the share instances using admin API
        replica_exports = self._remove_admin_only_exports(
            self.admin_client.list_share_instance_export_locations(
                replica['id'])['export_locations'])
        primary_replica_exports = self._remove_admin_only_exports(
            self.admin_client.list_share_instance_export_locations(
                primary_replica['id'])['export_locations'])

        return share, replica, primary_replica_exports, replica_exports

    def _validate_export_location_api_behavior(self, replica, replica_exports,
                                               primary_replica_exports,
                                               share_exports, version):
        share_export_paths = [e['path'] for e in share_exports]

        # Expectations
        expected_number_of_exports = len(primary_replica_exports
                                         + replica_exports)
        expected_exports = replica_exports + primary_replica_exports
        # In and beyond version 2.47, secondary "non-active" replica exports
        # are not expected to be present in the share export locations.
        # Secondary replicas can be "active" only in in "writable"
        # replication. In other types of replication, secondary replicas are
        # either "in_sync" or "out_of_sync"
        replica_is_non_active = (replica['replica_state'] !=
                                 constants.REPLICATION_STATE_ACTIVE)
        if utils.is_microversion_ge(version, '2.47') and replica_is_non_active:
            expected_number_of_exports = len(primary_replica_exports)
            expected_exports = primary_replica_exports

        # Assertions
        self.assertEqual(expected_number_of_exports, len(share_exports))
        for export in expected_exports:
            self.assertIn(export['path'], share_export_paths)

    @decorators.idempotent_id('da22cfb8-7dd8-4bf1-87fc-a1f7b51ebf8e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(*utils.deduplicate(['2.46', '2.47', LATEST_MICROVERSION]))
    def test_replicated_share_export_locations(self, version):
        """Test behavior changes in the share export locations API at 2.47"""
        utils.check_skip_if_microversion_not_supported(version)
        share, replica, primary_replica_exports, replica_exports = (
            self._create_share_and_replica_get_exports()
        )

        # Share export locations list API
        share_exports = self.shares_v2_client.list_share_export_locations(
            share['id'], version=version)['export_locations']

        self._validate_export_location_api_behavior(replica, replica_exports,
                                                    primary_replica_exports,
                                                    share_exports, version)

    @decorators.idempotent_id('58430f57-c6eb-44e2-9583-eecb1dd10594')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(*utils.deduplicate(['2.46', '2.47', LATEST_MICROVERSION]))
    @testtools.skipUnless(
        CONF.share.backend_replication_type in
        (constants.REPLICATION_STYLE_READABLE, constants.REPLICATION_STYLE_DR),
        'Promotion of secondary not supported in writable replication style.')
    def test_replicated_share_export_locations_with_promotion(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        share, replica, primary_replica_exports, replica_exports = (
            self._create_share_and_replica_get_exports(cleanup_replica=False)
        )
        primary_replica = self.shares_v2_client.get_share_replica(
            primary_replica_exports[0]['share_instance_id'])['share_replica']
        waiters.wait_for_resource_status(
            self.shares_v2_client, replica['id'],
            constants.REPLICATION_STATE_IN_SYNC, resource_name='share_replica',
            status_attr='replica_state')

        # Share export locations list API
        share_exports = self.shares_v2_client.list_share_export_locations(
            share['id'], version=version)['export_locations']

        # Validate API behavior
        self._validate_export_location_api_behavior(replica, replica_exports,
                                                    primary_replica_exports,
                                                    share_exports, version)

        # Promote share replica
        self.promote_share_replica(replica['id'])

        # Refresh for verification
        current_secondary_replica = self.shares_v2_client.get_share_replica(
            primary_replica['id'])['share_replica']
        current_primary_replica_exports = self._remove_admin_only_exports(
            self.admin_client.list_share_instance_export_locations(
                replica['id'], version=version)['export_locations'])
        current_secondary_replica_exports = self._remove_admin_only_exports(
            self.admin_client.list_share_instance_export_locations(
                primary_replica['id'], version=version)['export_locations'])
        share_exports = self.shares_v2_client.list_share_export_locations(
            share['id'], version=version)['export_locations']

        # Validate API behavior
        self._validate_export_location_api_behavior(
            current_secondary_replica, current_secondary_replica_exports,
            current_primary_replica_exports, share_exports, version)

        # Delete the secondary (the 'active' replica before promotion)
        self.delete_share_replica(primary_replica['id'])

    @decorators.idempotent_id('10ab6304-a1cd-4e60-90e3-7f9358b8808a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.47')
    def test_replica_export_locations(self):
        """Validates exports from the replica export locations APIs"""
        el_summary_keys = ['id', 'path', 'replica_state',
                           'availability_zone', 'preferred']
        el_detail_keys = el_summary_keys + ['created_at', 'updated_at']
        share, replica, expected_primary_exports, expected_replica_exports = (
            self._create_share_and_replica_get_exports()
        )
        primary_replica = self.shares_v2_client.get_share_replica(
            expected_primary_exports[0]['share_instance_id'])['share_replica']
        expected_primary_export_paths = [e['path'] for e in
                                         expected_primary_exports]
        expected_replica_export_paths = [e['path'] for e in
                                         expected_replica_exports]

        # For the primary replica
        actual_primary_exports = (
            self.shares_v2_client.list_share_replica_export_locations(
                primary_replica['id'])['export_locations']
        )

        self.assertEqual(len(expected_primary_exports),
                         len(actual_primary_exports))
        for export in actual_primary_exports:
            self.assertIn(export['path'], expected_primary_export_paths)
            self.assertEqual(constants.REPLICATION_STATE_ACTIVE,
                             export['replica_state'])
            self.assertEqual(share['availability_zone'],
                             export['availability_zone'])
            self.assertEqual(sorted(el_summary_keys), sorted(export.keys()))

            export_location_details = (
                self.shares_v2_client.get_share_replica_export_location(
                    primary_replica['id'], export['id'])['export_location']
            )
            self.assertEqual(sorted(el_detail_keys),
                             sorted(export_location_details.keys()))
            for key in el_summary_keys:
                self.assertEqual(export[key], export_location_details[key])

        # For the secondary replica
        actual_replica_exports = (
            self.shares_v2_client.list_share_replica_export_locations(
                replica['id'])['export_locations']
        )

        self.assertEqual(len(expected_replica_exports),
                         len(actual_replica_exports))
        for export in actual_replica_exports:
            self.assertIn(export['path'], expected_replica_export_paths)
            self.assertEqual(replica['replica_state'],
                             export['replica_state'])
            self.assertEqual(replica['availability_zone'],
                             export['availability_zone'])
            self.assertEqual(sorted(el_summary_keys), sorted(export.keys()))

            export_location_details = (
                self.shares_v2_client.get_share_replica_export_location(
                    replica['id'], export['id'])['export_location']
            )
            self.assertEqual(sorted(el_detail_keys),
                             sorted(export_location_details.keys()))
            for key in el_summary_keys:
                self.assertEqual(export[key], export_location_details[key])



# Copyright 2019 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ManageShareServersNegativeTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ManageShareServersNegativeTest, cls).skip_checks()
        if not CONF.share.multitenancy_enabled:
            raise cls.skipException('Multitenancy tests are disabled.')
        if not CONF.share.run_manage_unmanage_tests:
            raise cls.skipException('Manage/unmanage tests are disabled.')

        utils.check_skip_if_microversion_not_supported('2.49')

    @classmethod
    def resource_setup(cls):
        super(ManageShareServersNegativeTest, cls).resource_setup()

        # create share type
        cls.st_name = data_utils.rand_name("manage-st-name")
        cls.extra_specs = {
            'storage_protocol': CONF.share.capability_storage_protocol,
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        cls.share_type = cls.create_share_type(
            name=cls.st_name,
            cleanup_in_class=True,
            extra_specs=cls.extra_specs)
        cls.original_share_network = cls.shares_v2_client.get_share_network(
            cls.shares_v2_client.share_network_id)['share_network']
        cls.share_net_info = (
            utils.share_network_get_default_subnet(cls.original_share_network)
            if utils.share_network_subnets_are_supported() else
            cls.original_share_network)

    def _create_share_with_new_share_network(self):
        share_network = self.create_share_network(
            neutron_net_id=self.share_net_info['neutron_net_id'],
            neutron_subnet_id=self.share_net_info['neutron_subnet_id'],
            cleanup_in_class=True
        )
        share = self.create_share(
            share_type_id=self.share_type['id'],
            share_network_id=share_network['id']
        )
        return self.shares_v2_client.get_share(share['id'])['share']

    @ddt.data(
        ('host', 'invalid_host'),
        ('share_network_id', 'invalid_share_network_id'),
        ('share_network_subnet_id', 'invalid_share_network_subnet_id'),
    )
    @ddt.unpack
    @decorators.idempotent_id('60b94091-f35d-42ac-a25a-ca1f7e66bb44')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share_server_invalid_params(self, param, invalid_value):

        sn_subnets_not_supported = utils.is_microversion_lt(
            LATEST_MICROVERSION, utils.SHARE_NETWORK_SUBNETS_MICROVERSION)
        if param == 'share_network_subnet_id' and sn_subnets_not_supported:
            raise self.skipException("Share network subnets not supported by "
                                     "microversion %s" % LATEST_MICROVERSION)

        # create share
        share = self._create_share_with_new_share_network()
        el = self.shares_v2_client.list_share_export_locations(
            share['id'])['export_locations']
        share['export_locations'] = el
        share_server = self.shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']

        self._unmanage_share_and_wait(share)
        self._unmanage_share_server_and_wait(share_server)

        # forge invalid params
        invalid_params = share_server.copy()
        invalid_params[param] = invalid_value

        # try to manage in the wrong way
        self.assertRaises(
            lib_exc.BadRequest,
            self._manage_share_server,
            share_server,
            invalid_params
        )

        # manage in the correct way
        managed_share_server = self._manage_share_server(share_server)
        managed_share = self._manage_share(
            share,
            name="managed share that had ID %s" % share['id'],
            description="description for managed share",
            share_server_id=managed_share_server['id']
        )

        # delete share
        self._delete_share_and_wait(managed_share)

        # delete share server
        self._delete_share_server_and_wait(managed_share_server['id'])

    @decorators.idempotent_id('0f21aebe-e983-4112-a494-b287293112d2')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share_server_invalid_state(self):

        # create share
        share = self._create_share_with_new_share_network()

        for state in (constants.SERVER_STATE_MANAGE_STARTING,
                      constants.SERVER_STATE_CREATING,
                      constants.SERVER_STATE_DELETING):
            # leave it in the wrong state
            self.shares_v2_client.share_server_reset_state(
                share['share_server_id'],
                status=state,
            )

            # try to delete
            self.assertRaises(
                lib_exc.Forbidden,
                self.shares_v2_client.delete_share_server,
                share['share_server_id'],
            )

            # put it in the correct state
            self.shares_v2_client.share_server_reset_state(
                share['share_server_id'],
                status=constants.SERVER_STATE_ACTIVE,
            )
            waiters.wait_for_resource_status(
                self.shares_v2_client, share['share_server_id'],
                constants.SERVER_STATE_ACTIVE, resource_name='share_server'
            )

        # delete share
        self._delete_share_and_wait(share)

        # delete share server
        self._delete_share_server_and_wait(share['share_server_id'])

    @decorators.idempotent_id('84f9a808-8bff-4652-8877-4c9f0dfeed8a')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share_server_invalid_state(self):

        # create share
        share = self._create_share_with_new_share_network()

        for state in (constants.SERVER_STATE_MANAGE_STARTING,
                      constants.SERVER_STATE_CREATING,
                      constants.SERVER_STATE_DELETING):
            # leave it in the wrong state
            self.shares_v2_client.share_server_reset_state(
                share['share_server_id'],
                status=state,
            )

            # try to unmanage
            self.assertRaises(
                lib_exc.BadRequest,
                self.shares_v2_client.unmanage_share_server,
                share['share_server_id'],
            )

            # put it in the correct state
            self.shares_v2_client.share_server_reset_state(
                share['share_server_id'],
                status=constants.SERVER_STATE_ACTIVE,
            )
            waiters.wait_for_resource_status(
                self.shares_v2_client, share['share_server_id'],
                constants.SERVER_STATE_ACTIVE, resource_name='share_server'
            )

        # delete share
        self._delete_share_and_wait(share)

        # delete share server
        self._delete_share_server_and_wait(share['share_server_id'])

    @decorators.idempotent_id('fbe6cf2d-3e33-40d8-b420-b785d3cb7559')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_share_server_reset_state_invalid_state(self):

        # create share
        share = self.create_share(
            share_type_id=self.share_type['id'])
        share = self.shares_v2_client.get_share(share['id'])['share']

        # try to change it to wrong state
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.share_server_reset_state,
            share['share_server_id'],
            status='invalid_state',
        )

        # delete share
        self._delete_share_and_wait(share)

    @decorators.idempotent_id('c0a13fc4-9f30-483a-b473-0f0ce533f645')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share_server_with_share(self):

        # create share
        share = self.create_share(
            share_type_id=self.share_type['id'])
        share = self.shares_v2_client.get_share(share['id'])['share']

        # try to unmanage
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.unmanage_share_server,
            share['share_server_id'],
        )

        # delete share
        self._delete_share_and_wait(share)

    @decorators.idempotent_id('d2feb338-5c78-4370-b5ba-c8b16df63b43')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share_server_invalid_identifier(self):
        # create share
        share = self._create_share_with_new_share_network()
        el = self.shares_v2_client.list_share_export_locations(
            share['id'])['export_locations']
        share['export_locations'] = el
        share_server = self.shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']

        self._unmanage_share_and_wait(share)
        self._unmanage_share_server_and_wait(share_server)

        # forge invalid params
        invalid_params = share_server.copy()
        invalid_params['identifier'] = 'invalid_id'

        self.assertRaises(
            share_exceptions.ShareServerBuildErrorException,
            self._manage_share_server,
            invalid_params
        )

        # unmanage the share server in manage_error
        search_opts = {'identifier': 'invalid_id'}
        invalid_servers = self.shares_v2_client.list_share_servers(
            search_opts)['share_servers']
        self._unmanage_share_server_and_wait(invalid_servers[0])

        # manage in the correct way
        managed_share_server = self._manage_share_server(share_server)
        managed_share_server = self.shares_v2_client.show_share_server(
            managed_share_server['id']
        )['share_server']
        managed_share = self._manage_share(
            share,
            name="managed share that had ID %s" % share['id'],
            description="description for managed share",
            share_server_id=managed_share_server['id']
        )

        # delete share
        self._delete_share_and_wait(managed_share)

        # delete share server
        self._delete_share_server_and_wait(managed_share_server['id'])

    @decorators.idempotent_id('30606085-4b01-4489-a18c-22e67ed0e221')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share_server_double_manage(self):

        # create share
        share = self.create_share(
            share_type_id=self.share_type['id'])
        share = self.shares_v2_client.get_share(share['id'])['share']

        share_server = self.shares_v2_client.show_share_server(
            share['share_server_id'])['share_server']

        # try with more data around the identifier
        invalid_params = share_server.copy()
        invalid_params['identifier'] = (
            'foo_' + share_server['identifier'] + '_bar')

        self.assertRaises(
            lib_exc.BadRequest,
            self._manage_share_server,
            invalid_params)

        # try with part of the identifier
        invalid_params['identifier'] = (
            share_server['identifier'].split("-")[-1])

        self.assertRaises(
            lib_exc.BadRequest,
            self._manage_share_server,
            invalid_params)

        # try with same identifier but underscores
        invalid_params['identifier'] = (
            share_server['identifier'].replace("-", "_"))

        self.assertRaises(
            lib_exc.BadRequest,
            self._manage_share_server,
            invalid_params)

        # delete share
        self._delete_share_and_wait(share)

        # Delete share server, since it can't be "auto-deleted"
        if not CONF.share.share_network_id:
            # For a pre-configured share_network_id, we don't
            # delete the share server.
            self._delete_share_server_and_wait(share_server['id'])



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareNetworksNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(ShareNetworksNegativeTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

    @decorators.idempotent_id('66289664-bf01-40dd-a76d-fd2c953bbceb')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_share_network_without_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_share_network, "")

    @decorators.idempotent_id('80397850-2f64-48b3-b19b-79c4ac0bd58f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_share_network_with_wrong_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_share_network, "wrong_id")

    @decorators.idempotent_id('fe6ac194-5003-404c-b372-7515a58ff969')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_share_network_without_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.delete_share_network, "")

    @decorators.idempotent_id('7e22e8b9-a1ce-480c-89b3-b4edd807285a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_share_network_with_wrong_type(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.delete_share_network, "wrong_id")

    @decorators.idempotent_id('a7c55dbe-c23e-403f-b8aa-4aa1128f32a4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_nonexistant_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.update_share_network,
                          "wrong_id", name="name")

    @decorators.idempotent_id('984349ca-df7d-4f85-a45f-948189debb65')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_share_network_with_empty_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.update_share_network,
                          "", name="name")

    @decorators.idempotent_id('211b64b4-4c2b-4b6b-b011-725f40a37b03')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_try_update_invalid_keys_sh_server_exists(self):
        self.create_share(share_type_id=self.share_type_id,
                          cleanup_in_class=False)

        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.update_share_network,
                          self.shares_client.share_network_id,
                          neutron_net_id="new_net_id")

    @decorators.idempotent_id('9166b81c-d6ab-4592-bcf7-9410250e30dd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_deleted_share_network(self):
        data = utils.generate_share_network_data()
        sn = self.create_share_network(**data)
        self.assertDictContainsSubset(data, sn)

        self.shares_client.delete_share_network(sn["id"])

        # try get deleted share network entity
        self.assertRaises(lib_exc.NotFound,
                          self.shares_client.get_security_service,
                          sn["id"])

    @decorators.idempotent_id('0d104b72-aab5-48b5-87f8-847d2155faa9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_share_networks_wrong_created_since_value(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_client.list_share_networks_with_detail,
            params={'created_since': '2014-10-23T08:31:58.000000'})

    @decorators.idempotent_id('c96dacaf-4cea-4fe9-bbd7-c9b1001f5495')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_share_networks_wrong_created_before_value(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_client.list_share_networks_with_detail,
            params={'created_before': '2014-10-23T08:31:58.000000'})

    @decorators.idempotent_id('6e4912fd-ae85-4a43-81e8-e5b340099b64')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(not CONF.share.multitenancy_enabled,
                      'Can run only with drivers that do handle share servers '
                      'creation. Skipping.')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    def test_try_delete_share_network_with_existing_shares(self):
        # Get valid network data for successful share creation
        share_network = self.shares_client.get_share_network(
            self.shares_client.share_network_id)['share_network']
        new_sn = self.create_share_network(
            neutron_net_id=share_network['neutron_net_id'],
            neutron_subnet_id=share_network['neutron_subnet_id'],
            cleanup_in_class=False)

        # Create share with share network
        self.create_share(share_type_id=self.share_type_id,
                          share_network_id=new_sn['id'],
                          cleanup_in_class=False)

        # Try delete share network
        self.assertRaises(
            lib_exc.Conflict,
            self.shares_client.delete_share_network, new_sn['id'])

    @decorators.idempotent_id('4e71de31-1064-40da-948d-a72063fbd647')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.35")
    def test_list_shares_with_like_filter_not_exist(self):
        filters = {
            'name~': 'fake_not_exist',
            'description~': 'fake_not_exist',
        }
        share_networks = (
            self.shares_v2_client.list_share_networks_with_detail(
                params=filters)['share_networks'])

        self.assertEqual(0, len(share_networks))

    @utils.skip_if_microversion_not_supported("2.51")
    @decorators.idempotent_id('8a995305-ede9-4002-a9cd-f24ff4d71f63')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share_network_contains_more_than_one_subnet(self):
        share_network = self.create_share_network()
        az = self.shares_v2_client.list_availability_zones(
            )['availability_zones'][0]
        az_name = az['name']

        # Generate subnet data
        data = utils.generate_subnet_data()
        data['share_network_id'] = share_network['id']
        data['availability_zone'] = az_name

        # create share network
        subnet = self.create_share_network_subnet(**data)

        # Try to delete the share network
        self.assertRaises(
            lib_exc.Conflict,
            self.shares_client.delete_share_network,
            share_network['id']
        )

        self.shares_v2_client.delete_subnet(share_network['id'], subnet['id'])
        share_network = self.shares_v2_client.get_share_network(
            share_network['id'])['share_network']
        default_subnet = share_network['share_network_subnets'][0]
        self.assertIsNone(default_subnet['availability_zone'])

    @utils.skip_if_microversion_not_supported("2.51")
    @decorators.idempotent_id('d84c3c5c-5913-42d4-9a66-0d5a78295adb')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_share_network_inexistent_az(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.shares_v2_client.create_share_network,
            availability_zone='inexistent-availability-zone',
        )

    @utils.skip_if_microversion_not_supported("2.70")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @decorators.idempotent_id('f6f47c64-6821-4d4a-aa7d-3b0244158197')
    def test_check_add_share_network_subnet_share_network_not_found(self):
        data = utils.generate_subnet_data()
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.subnet_create_check,
                          'fake_inexistent_id',
                          **data)

    @utils.skip_if_microversion_not_supported("2.70")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @decorators.idempotent_id('d9a487fb-6638-4f93-8b69-3e1a85bfbc7d')
    def test_check_add_share_network_subnet_az_not_found(self):
        share_network = self.create_share_network()
        data = {'availability_zone': 'non-existent-az'}

        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.subnet_create_check,
                          share_network['id'], **data)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base

CONF = config.CONF


@ddt.ddt
class SharesQuotasNegativeTest(base.BaseSharesTest):

    @classmethod
    def skip_checks(cls):
        super(SharesQuotasNegativeTest, cls).skip_checks()
        if not CONF.share.run_quota_tests:
            msg = "Quota tests are disabled."
            raise cls.skipException(msg)

    @decorators.idempotent_id('d0dfe81d-8e8c-4847-a55f-95ba8a3d922c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_quotas_with_empty_tenant_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.show_quotas, "")

    @decorators.idempotent_id('e7dbc580-1857-4f88-8886-988dc2f2c7b9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_reset_quotas_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.reset_quotas,
                          self.shares_v2_client.tenant_id)

    @decorators.idempotent_id('f1c8e16f-5406-4389-a29c-547cca8a56e0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_quotas_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_v2_client.update_quotas,
                          self.shares_v2_client.tenant_id,
                          shares=9)

    @ddt.data("2.6", "2.7", "2.24")
    @decorators.idempotent_id('0f0033b3-357e-42e6-9c94-cac650e1cd50')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_quotas_detail_with_wrong_version(self, microversion):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.detail_quotas,
                          self.shares_v2_client.tenant_id,
                          version=microversion)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


@ddt.ddt
class ServicesAdminNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(ServicesAdminNegativeTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.shares_v2_client

    @decorators.idempotent_id('59ab1a7d-817d-42e5-a1b9-adfc2912e08c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_services_with_non_admin_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.member_client.list_services)

    @decorators.idempotent_id('04f59090-069d-4cae-b028-e004ff29126c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_service_by_invalid_params(self):
        # All services are expected if send the request with invalid parameter
        services = self.admin_client.list_services()['services']
        params = {'fake_param': 'fake_param_value'}
        services_fake = self.admin_client.list_services(params)['services']
        self.assertEqual(len(services), len(services_fake))

        # "update_at" field could be updated before second request,
        # so do not take it in account.
        for service in services + services_fake:
            service["updated_at"] = "removed_possible_difference"

        msg = ('Unexpected service list. Expected %s, got %s.' %
               (services, services_fake))
        self.assertEqual(sorted(services, key=lambda service: service['id']),
                         sorted(services_fake,
                                key=lambda service: service['id']),
                         msg)

    @decorators.idempotent_id('386dc131-39f9-4d5f-af87-edf76e3d0e71')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_service_by_invalid_host(self):
        params = {'host': 'fake_host'}
        services_fake = self.admin_client.list_services(params)['services']
        self.assertEqual(0, len(services_fake))

    @decorators.idempotent_id('766461b0-e89a-4113-8229-24c4d11d585a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_service_by_invalid_binary(self):
        params = {'binary': 'fake_binary'}
        services_fake = self.admin_client.list_services(params)['services']
        self.assertEqual(0, len(services_fake))

    @decorators.idempotent_id('ac570fde-690d-4448-9cce-ce35e0a14b88')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_service_by_invalid_zone(self):
        params = {'zone': 'fake_zone'}
        services_fake = self.admin_client.list_services(params)['services']
        self.assertEqual(0, len(services_fake))

    @decorators.idempotent_id('da0fef1d-c4d3-4c33-a836-5f836e85df69')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_service_by_invalid_status(self):
        params = {'status': 'fake_status'}
        services_fake = self.admin_client.list_services(params)['services']
        self.assertEqual(0, len(services_fake))

    @decorators.idempotent_id('41936575-3a96-455b-8069-7f6563abf0e2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_service_by_invalid_state(self):
        params = {'state': 'fake_state'}
        services_fake = self.admin_client.list_services(params)['services']
        self.assertEqual(0, len(services_fake))

    @decorators.idempotent_id('3c72227b-7fa1-4294-bdf4-413ec4c324e5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data(
        ('os-services', '2.7'),
        ('services', '2.6'),
        ('services', '2.0'),
    )
    @ddt.unpack
    @utils.skip_if_microversion_not_supported("2.7")
    def test_list_services_with_wrong_versions(self, url, version):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.list_services,
            version=version, url=url,
        )



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ShareGroupsTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ShareGroupsTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareGroupsTest, cls).resource_setup()
        # Create 2 share_types
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        cls.share_type2 = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id2 = cls.share_type2['id']

        # Create a share group type
        name = data_utils.rand_name("unique_sgt_name")
        cls.sg_type = cls.create_share_group_type(
            name=name,
            share_types=[cls.share_type_id, cls.share_type_id2],
            cleanup_in_class=True,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)
        cls.sg_type_id = cls.sg_type['id']

    @decorators.idempotent_id('79eaa86f-4c8f-49fd-acb2-ec051aa6bf93')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_create_share_group_with_single_share_type_min(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        share_group = self.create_share_group(
            share_group_type_id=self.sg_type_id,
            cleanup_in_class=False,
            share_type_ids=[self.share_type_id],
            version=version)

        keys = set(share_group.keys())
        self.assertTrue(
            constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS.issubset(keys),
            'At least one expected element missing from share group '
            'response. Expected %(expected)s, got %(actual)s.' % {
                "expected": constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS,
                "actual": keys})

        actual_sg_type = share_group['share_group_type_id']
        expected_sg_type = self.sg_type_id
        self.assertEqual(
            expected_sg_type, actual_sg_type,
            'Incorrect share group type applied to share group '
            '%s. Expected %s, got %s' % (
                share_group['id'], expected_sg_type, actual_sg_type))

        actual_share_types = share_group['share_types']
        expected_share_types = [self.share_type_id]
        self.assertEqual(
            sorted(expected_share_types),
            sorted(actual_share_types),
            'Incorrect share types applied to share group %s. '
            'Expected %s, got %s' % (
                share_group['id'], expected_share_types, actual_share_types))

    @decorators.idempotent_id('ff6c17af-03ba-4506-923d-b6c229492d0e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_share_group_with_multiple_share_types_min(self):
        share_group = self.create_share_group(
            share_group_type_id=self.sg_type_id,
            cleanup_in_class=False,
            share_type_ids=[self.share_type_id, self.share_type_id2],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        keys = set(share_group.keys())
        self.assertTrue(
            constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS.issubset(keys),
            'At least one expected element missing from share group '
            'response. Expected %(expected)s, got %(actual)s.' % {
                "expected": constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS,
                "actual": keys})

        actual_sg_type = share_group['share_group_type_id']
        expected_sg_type = self.sg_type_id
        self.assertEqual(
            expected_sg_type, actual_sg_type,
            'Incorrect share group type applied to share group %s. '
            'Expected %s, got %s' % (
                share_group['id'], expected_sg_type, actual_sg_type))

        actual_share_types = share_group['share_types']
        expected_share_types = [self.share_type_id, self.share_type_id2]
        self.assertEqual(
            sorted(expected_share_types),
            sorted(actual_share_types),
            'Incorrect share types applied to share group %s. '
            'Expected %s, got %s' % (
                share_group['id'], expected_share_types, actual_share_types))

    @decorators.idempotent_id('99f0471c-e978-42ac-b50b-848b16692eab')
    @testtools.skipUnless(
        CONF.share.default_share_type_name, "Only if defaults are defined.")
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_default_share_group_type_applied(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        try:
            default_type = self.shares_v2_client.get_default_share_group_type(
                version=version
            )['share_group_type']
        except exceptions.NotFound:
            msg = "There is no default share group type"
            raise self.skipException(msg)
        default_share_types = default_type['share_types']

        share_group = self.create_share_group(
            cleanup_in_class=False,
            share_type_ids=default_share_types,
            version=version)

        keys = set(share_group.keys())
        self.assertTrue(
            constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS.issubset(keys),
            'At least one expected element missing from share group '
            'response. Expected %(expected)s, got %(actual)s.' % {
                "expected": constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS,
                "actual": keys})

        actual_sg_type = share_group['share_group_type_id']
        expected_sg_type = default_type['id']
        self.assertEqual(
            expected_sg_type, actual_sg_type,
            'Incorrect share group type applied to share group %s. '
            'Expected %s, got %s' % (
                share_group['id'], expected_sg_type, actual_sg_type))

    @decorators.idempotent_id('8ca1f0a0-2a36-4adb-af6b-6741b00307c5')
    @testtools.skipUnless(
        CONF.share.multitenancy_enabled, "Only for multitenancy.")
    @testtools.skipUnless(
        CONF.share.run_snapshot_tests, "Snapshot tests are disabled.")
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_from_snapshot_verify_share_server_information_min(self):
        # Create a share group
        orig_sg = self.create_share_group(
            share_group_type_id=self.sg_type_id,
            cleanup_in_class=False,
            share_type_ids=[self.share_type_id],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        # Get latest share group information
        orig_sg = self.shares_v2_client.get_share_group(
            orig_sg['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)['share_group']

        # Assert share server information
        self.assertIsNotNone(orig_sg['share_network_id'])
        self.assertIsNotNone(orig_sg['share_server_id'])

        sg_snapshot = self.create_share_group_snapshot_wait_for_active(
            orig_sg['id'], cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)
        new_sg = self.create_share_group(
            share_group_type_id=self.sg_type_id,
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
            source_share_group_snapshot_id=sg_snapshot['id'])

        # Assert share server information
        self.assertEqual(
            orig_sg['share_network_id'], new_sg['share_network_id'])
        self.assertEqual(
            orig_sg['share_server_id'], new_sg['share_server_id'])

    @decorators.idempotent_id('93fd4a97-a25a-4a17-b5ae-c8894c1adfc5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_with_sg_type_but_without_any_group_specs(self):
        # Create share group type not specifying any group specs
        sg_type = self.create_share_group_type(
            name=data_utils.rand_name("tempest-manila"),
            share_types=[self.share_type_id],
            group_specs={},
            cleanup_in_class=False)

        # Create share group, it should be created always, because we do not
        # restrict choice anyhow.
        self.create_share_group(
            share_type_ids=[self.share_type_id],
            share_group_type_id=sg_type['id'],
            cleanup_in_class=False)



# Copyright 2015 mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


class AvailabilityZonesTest(base.BaseSharesTest):

    def _list_availability_zones_assertions(self, availability_zones):
        self.assertGreater(len(availability_zones), 0)
        keys = ("created_at", "updated_at", "name", "id")
        for az in availability_zones:
            self.assertEqual(len(keys), len(az))
            for key in keys:
                self.assertIn(key, az)

    @decorators.idempotent_id('202f20d3-1afa-40ea-a5e6-8b7bda40e6cf')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_availability_zones_legacy_url_api_v1(self):
        # NOTE(vponomaryov): remove this test with removal of availability zone
        # extension url support.
        azs = self.shares_client.list_availability_zones(
            )['availability_zones']
        self._list_availability_zones_assertions(azs)

    @decorators.idempotent_id('7054f2f4-bc77-4d60-82a6-2f23b93d281e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.6")
    def test_list_availability_zones_legacy_url_api_v2(self):
        # NOTE(vponomaryov): remove this test with removal of availability zone
        # extension url support.
        azs = self.shares_v2_client.list_availability_zones(
            url='os-availability-zone', version='2.6')['availability_zones']
        self._list_availability_zones_assertions(azs)

    @decorators.idempotent_id('4caadb86-2988-4adb-b705-aece99235c1e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.7")
    def test_list_availability_zones(self):
        azs = self.shares_v2_client.list_availability_zones(
            version='2.7')['availability_zones']
        self._list_availability_zones_assertions(azs)



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF


@ddt.ddt
class RevertToSnapshotNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(RevertToSnapshotNegativeTest, cls).skip_checks()
        if not CONF.share.run_revert_to_snapshot_tests:
            msg = "Revert to snapshot tests are disabled."
            raise cls.skipException(msg)
        if not CONF.share.capability_snapshot_support:
            msg = "Snapshot support is disabled."
            raise cls.skipException(msg)
        if not CONF.share.run_snapshot_tests:
            msg = "Snapshot tests are disabled."
            raise cls.skipException(msg)

        utils.check_skip_if_microversion_not_supported(
            constants.REVERT_TO_SNAPSHOT_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(RevertToSnapshotNegativeTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        pools = cls.admin_client.list_pools(detail=True)['pools']
        revert_support = [
            pool['capabilities'][constants.REVERT_TO_SNAPSHOT_SUPPORT]
            for pool in pools]
        if not any(revert_support):
            msg = "Revert to snapshot not supported."
            raise cls.skipException(msg)

        cls.share_type_name = data_utils.rand_name("share-type")
        extra_specs = {
            "snapshot_support": True,
            constants.REVERT_TO_SNAPSHOT_SUPPORT: True
        }
        cls.revert_enabled_extra_specs = cls.add_extra_specs_to_dict(
            extra_specs=extra_specs)

        cls.share_type = cls.create_share_type(
            cls.share_type_name,
            extra_specs=cls.revert_enabled_extra_specs,
            client=cls.admin_client)

        cls.st_id = cls.share_type['id']

        cls.share = cls.create_share(share_type_id=cls.st_id)
        cls.share2 = cls.create_share(share_type_id=cls.st_id)

    @decorators.idempotent_id('21dd8561-8913-42a2-a95c-74b536964c94')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_to_second_latest_snapshot(self, version):
        snapshot1 = self.create_snapshot_wait_for_active(
            self.share['id'], cleanup_in_class=False)
        self.create_snapshot_wait_for_active(self.share['id'],
                                             cleanup_in_class=False)

        self.assertRaises(exceptions.Conflict,
                          self.shares_v2_client.revert_to_snapshot,
                          self.share['id'],
                          snapshot1['id'],
                          version=version)

    @decorators.idempotent_id('7360ee16-ac7d-46ce-9c81-251d64fb7434')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_to_error_snapshot(self, version):
        snapshot = self.create_snapshot_wait_for_active(self.share['id'],
                                                        cleanup_in_class=False)

        self.admin_client.reset_state(snapshot['id'],
                                      status=constants.STATUS_ERROR,
                                      s_type='snapshots')

        self.assertRaises(exceptions.Conflict,
                          self.shares_v2_client.revert_to_snapshot,
                          self.share['id'],
                          snapshot['id'],
                          version=version)

    @decorators.idempotent_id('108a451e-a8e9-450f-8f75-53883d58c6be')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_error_share_to_snapshot(self, version):
        snapshot = self.create_snapshot_wait_for_active(self.share['id'],
                                                        cleanup_in_class=False)

        self.admin_client.reset_state(self.share['id'],
                                      status=constants.STATUS_ERROR,
                                      s_type='shares')

        self.addCleanup(self.admin_client.reset_state,
                        self.share['id'],
                        status=constants.STATUS_AVAILABLE,
                        s_type='shares')

        self.assertRaises(exceptions.Conflict,
                          self.shares_v2_client.revert_to_snapshot,
                          self.share['id'],
                          snapshot['id'],
                          version=version)

    @decorators.idempotent_id('29024057-dbbd-4cf6-a796-207dfbd4c3ff')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_to_missing_snapshot(self, version):
        self.assertRaises(exceptions.BadRequest,
                          self.shares_v2_client.revert_to_snapshot,
                          self.share['id'],
                          self.share['id'],
                          version=version)

    @decorators.idempotent_id('ef706fad-5ac4-41dc-af81-5aa0331560cf')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.REVERT_TO_SNAPSHOT_MICROVERSION,
                            CONF.share.max_api_microversion])
    )
    def test_revert_to_invalid_snapshot(self, version):
        snapshot = self.create_snapshot_wait_for_active(
            self.share['id'], cleanup_in_class=False)

        self.assertRaises(exceptions.BadRequest,
                          self.shares_v2_client.revert_to_snapshot,
                          self.share2['id'],
                          snapshot['id'],
                          version=version)



# Copyright (c) 2017 Hitachi Data Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from oslo_utils import uuidutils
from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class SnapshotExportLocationsTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(SnapshotExportLocationsTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')
        if not CONF.share.run_mount_snapshot_tests:
            raise cls.skipException('Mountable snapshots tests are disabled.')

        utils.check_skip_if_microversion_not_supported("2.32")

    @classmethod
    def setup_clients(cls):
        super(SnapshotExportLocationsTest, cls).setup_clients()
        cls.admin_client = cls.admin_shares_v2_client

    @classmethod
    def resource_setup(cls):
        super(SnapshotExportLocationsTest, cls).resource_setup()
        # create share type
        extra_specs = {
            'snapshot_support': True,
            'mount_snapshot_support': True,
        }
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id,
                                     client=cls.admin_client)
        cls.snapshot = cls.create_snapshot_wait_for_active(
            cls.share['id'], client=cls.admin_client)
        cls.snapshot = cls.admin_client.get_snapshot(
            cls.snapshot['id'])['snapshot']
        cls.snapshot_instances = cls.admin_client.list_snapshot_instances(
            snapshot_id=cls.snapshot['id'])['snapshot_instances']

    def _verify_export_location_structure(
            self, export_locations, role='admin', detail=False):

        # Determine which keys to expect based on role, version and format
        summary_keys = ['id', 'path', 'links']
        if detail:
            summary_keys.extend(['created_at', 'updated_at'])

        admin_summary_keys = summary_keys + [
            'share_snapshot_instance_id', 'is_admin_only']

        if role == 'admin':
            expected_keys = admin_summary_keys
        else:
            expected_keys = summary_keys

        if not isinstance(export_locations, (list, tuple, set)):
            export_locations = (export_locations, )

        for export_location in export_locations:

            # Check that the correct keys are present
            self.assertEqual(len(expected_keys), len(export_location))
            for key in expected_keys:
                self.assertIn(key, export_location)

            # Check the format of ever-present summary keys
            self.assertTrue(uuidutils.is_uuid_like(export_location['id']))
            self.assertIsInstance(export_location['path'],
                                  str)

            if role == 'admin':
                self.assertIn(export_location['is_admin_only'], (True, False))
                self.assertTrue(uuidutils.is_uuid_like(
                    export_location['share_snapshot_instance_id']))

    @decorators.idempotent_id('18287f50-0e12-463d-906f-5c7cba256288')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_snapshot_export_location(self):
        export_locations = (
            self.admin_client.list_snapshot_export_locations(
                self.snapshot['id']))['share_snapshot_export_locations']

        for el in export_locations:
            self._verify_export_location_structure(el)

    @decorators.idempotent_id('6272b60b-31a1-41c1-86f5-af28926898e6')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_snapshot_export_location(self):
        export_locations = (
            self.admin_client.list_snapshot_export_locations(
                self.snapshot['id']))['share_snapshot_export_locations']

        for export_location in export_locations:
            el = self.admin_client.get_snapshot_export_location(
                self.snapshot['id'],
                export_location['id'])['share_snapshot_export_location']
            self._verify_export_location_structure(el, detail=True)

    @decorators.idempotent_id('03be6418-5ba3-4919-a798-89d7e5ffb925')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_snapshot_instance_export_location(self):
        for snapshot_instance in self.snapshot_instances:
            export_locations = (
                self.admin_client.list_snapshot_instance_export_locations(
                    snapshot_instance['id'])['share_snapshot_export_locations']
            )
            for el in export_locations:
                el = (
                    self.admin_client.get_snapshot_instance_export_location(
                        snapshot_instance['id'],
                        el['id'])['share_snapshot_export_location']
                )
                self._verify_export_location_structure(el, detail=True)

    @decorators.idempotent_id('cdf444ea-95a3-4f7b-ae48-6b027a6b9529')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_snapshot_contains_all_export_locations_of_all_snapshot_instances(
            self):
        snapshot_export_locations = (
            self.admin_client.list_snapshot_export_locations(
                self.snapshot['id']))['share_snapshot_export_locations']
        snapshot_instances_export_locations = []
        for snapshot_instance in self.snapshot_instances:
            snapshot_instance_export_locations = (
                self.admin_client.list_snapshot_instance_export_locations(
                    snapshot_instance['id'])['share_snapshot_export_locations']
            )
            snapshot_instances_export_locations.extend(
                snapshot_instance_export_locations)

        self.assertEqual(
            len(snapshot_export_locations),
            len(snapshot_instances_export_locations)
        )
        self.assertEqual(
            sorted(snapshot_export_locations, key=lambda el: el['id']),
            sorted(snapshot_instances_export_locations,
                   key=lambda el: el['id'])
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import ddt
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


@ddt.ddt
class ShareTypeAvailabilityZonesNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareTypeAvailabilityZonesNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported('2.48')

    @classmethod
    def resource_setup(cls):
        super(ShareTypeAvailabilityZonesNegativeTest, cls).resource_setup()
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']
        all_azs = cls.get_availability_zones()
        cls.valid_azs = cls.get_availability_zones_matching_share_type(
            cls.share_type)
        cls.invalid_azs = ((set(all_azs) - set(cls.valid_azs))
                           or ['az_that_doesnt_exist'])
        cls.az_spec = 'availability_zones'
        cls.valid_azs_spec = ', '.join(cls.valid_azs)
        cls.invalid_azs_spec = ', '.join(cls.invalid_azs)

    @decorators.idempotent_id('7a23af8d-a995-43ac-a381-2cd5c5f57f39')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('az1,     az2, az 3,  ', 'az1,,az 3', ',az2,  az 3')
    def test_share_type_azs_create_with_invalid_az_spec(self, spec):
        az_spec = {'availability_zones': spec}
        extra_specs = self.add_extra_specs_to_dict(az_spec)

        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_type,
            data_utils.rand_name('share_type_invalid_az_spec'),
            cleanup_in_class=False,
            extra_specs=extra_specs,
            client=self.admin_shares_v2_client)

    @decorators.idempotent_id('78f1afba-6087-415c-aaa1-a880ae341bfc')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_share_type_azs_filter_by_invalid_azs_extra_spec(self):
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type_id, self.az_spec, self.valid_azs_spec)
        share_type_no_az_spec = self.create_share_type(
            data_utils.rand_name('support_any_az_share_type'),
            cleanup_in_class=False,
            extra_specs=self.add_extra_specs_to_dict(),
            client=self.admin_shares_v2_client)

        share_types = self.admin_shares_v2_client.list_share_types(params={
            'extra_specs': {'availability_zones': self.invalid_azs_spec}}
        )['share_types']
        share_type_ids = [s['id'] for s in share_types]
        self.assertNotIn(self.share_type_id, share_type_ids)
        self.assertIn(share_type_no_az_spec['id'], share_type_ids)

    @decorators.idempotent_id('9285d7d5-cb40-462d-998c-891f0f934fdd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_share_type_azs_shares_unsupported_az(self):
        """Test using an AZ not supported by the share type."""
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type_id, self.az_spec, self.invalid_azs_spec)
        self.assertRaises(
            lib_exc.BadRequest, self.create_share,
            share_type_id=self.share_type_id,
            availability_zone=self.valid_azs[0],
            cleanup_in_class=False)

    @decorators.idempotent_id('5712abae-52a5-46ec-8850-46e732b18b06')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_share_type_azs_share_groups_unsupported(self):
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type_id, self.az_spec, self.invalid_azs_spec)
        self.assertRaises(
            lib_exc.BadRequest, self.create_share_group,
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
            availability_zone=self.valid_azs[0],
            cleanup_in_class=False)



# Copyright 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ManageNFSSnapshotNegativeTest(base.BaseSharesAdminTest):
    protocol = 'nfs'

    @classmethod
    @utils.skip_if_microversion_not_supported("2.12")
    @testtools.skipUnless(
        CONF.share.run_manage_unmanage_snapshot_tests,
        "Manage/unmanage snapshot tests are disabled.")
    def resource_setup(cls):
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

        utils.skip_if_manage_not_supported_for_version()

        super(ManageNFSSnapshotNegativeTest, cls).resource_setup()

        # Create share type
        cls.st_name = data_utils.rand_name("tempest-manage-st-name")
        cls.extra_specs = {
            'storage_protocol': CONF.share.capability_storage_protocol,
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
            'snapshot_support': str(
                CONF.share.capability_snapshot_support),
        }

        cls.st = cls.create_share_type(
            name=cls.st_name,
            cleanup_in_class=True,
            extra_specs=cls.extra_specs)

        # Create share
        cls.share = cls.create_share(
            share_type_id=cls.st['id'],
            share_protocol=cls.protocol
        )

    @decorators.idempotent_id('df7ac126-2c13-4902-b9c0-d103c71680ee')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_not_found(self):
        # Manage non-existing snapshot fails
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.manage_snapshot,
            'fake-share-id',
            'fake-provider-location',
        )

    @decorators.idempotent_id('645a88f7-1eec-49a2-a232-757dc2112dd8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_already_exists(self):
        # Manage already existing snapshot fails

        # Create snapshot
        snap = self.create_snapshot_wait_for_active(self.share['id'])
        snap = self.shares_v2_client.get_snapshot(
            snap['id'])['snapshot']
        self.assertEqual(self.share['id'], snap['share_id'])
        self.assertIsNotNone(snap['provider_location'])

        # Manage snapshot fails
        self.assertRaises(
            lib_exc.Conflict,
            self.shares_v2_client.manage_snapshot,
            self.share['id'],
            snap['provider_location']
        )

        # Delete snapshot
        self._delete_snapshot_and_wait(snap)

    @decorators.idempotent_id('e94706f4-9c72-41f1-9ed4-09eb93d0b36f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_invalid_provider_location(self):
        # Manage a snapshot with wrong provider location fails

        # Create snapshot
        snap = self.create_snapshot_wait_for_active(self.share['id'])
        snap = self.shares_v2_client.get_snapshot(
            snap['id'])['snapshot']

        # Unmanage snapshot
        self.shares_v2_client.unmanage_snapshot(snap['id'])
        self.shares_client.wait_for_resource_deletion(
            snapshot_id=snap['id']
        )

        # Manage snapshot with invalid provider location leaves it in
        # manage_error state
        invalid_snap = self.shares_v2_client.manage_snapshot(
            self.share['id'],
            'invalid_provider_location',
            driver_options={}
        )['snapshot']
        waiters.wait_for_resource_status(
            self.shares_v2_client, invalid_snap['id'],
            constants.STATUS_MANAGE_ERROR, resource_name='snapshot'
        )
        self.shares_v2_client.unmanage_snapshot(invalid_snap['id'])

        # Manage it properly and delete
        managed_snap = self.shares_v2_client.manage_snapshot(
            self.share['id'],
            snap['provider_location']
        )['snapshot']
        waiters.wait_for_resource_status(
            self.shares_v2_client, managed_snap['id'],
            constants.STATUS_AVAILABLE, resource_name='snapshot'
        )
        self._delete_snapshot_and_wait(managed_snap)

    @decorators.idempotent_id('d0049626-a524-4c8a-a58a-006ae2cdfea5')
    @testtools.skipUnless(CONF.share.multitenancy_enabled,
                          'Multitenancy tests are disabled.')
    @utils.skip_if_microversion_not_supported("2.48")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_snapshot_with_server_unsupported(self):
        share = self._create_share_for_manage()
        snap = self.create_snapshot_wait_for_active(share["id"])

        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.unmanage_snapshot,
            snap['id'], version="2.48")

        self._delete_snapshot_and_wait(snap)
        self._delete_share_and_wait(share)


class ManageCIFSSnapshotNegativeTest(ManageNFSSnapshotNegativeTest):
    protocol = 'cifs'


class ManageGLUSTERFSSnapshotNegativeTest(ManageNFSSnapshotNegativeTest):
    protocol = 'glusterfs'


class ManageHDFSSnapshotNegativeTest(ManageNFSSnapshotNegativeTest):
    protocol = 'hdfs'


class ManageMapRFSSnapshotNegativeTest(ManageNFSSnapshotNegativeTest):
    protocol = 'maprfs'
